"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getV1Releases = void 0;
const assign_keys_1 = require("../../../util/assign-keys");
const http_1 = require("../../../util/http");
const result_1 = require("../../../util/result");
const url_1 = require("../../../util/url");
const schema_1 = require("./schema");
function getV1Releases(http, registryUrl, packageName) {
    const fileName = `${packageName}.json`;
    const versionsUrl = (0, url_1.joinUrlParts)(registryUrl, '/api/v1/versions', fileName);
    const metadataUrl = (0, url_1.joinUrlParts)(registryUrl, '/api/v1/gems', fileName);
    const addMetadata = (releases) => http
        .getJsonSafe(metadataUrl, schema_1.GemMetadata)
        .transform((metadata) => (0, assign_keys_1.assignKeys)({ releases }, metadata, [
        'changelogUrl',
        'sourceUrl',
        'homepage',
    ]))
        .unwrap({ releases });
    return http
        .getJsonSafe(versionsUrl, schema_1.GemVersions)
        .catch((err) => {
        if (err instanceof http_1.HttpError) {
            const status = err.response?.statusCode;
            if (status === 404 || status === 400) {
                return result_1.Result.err('unsupported-api');
            }
        }
        return result_1.Result.err(err);
    })
        .transform((releases) => {
        return releases.length > 0
            ? result_1.Result.ok(releases)
            : result_1.Result.err('empty-releases');
    })
        .transform(addMetadata);
}
exports.getV1Releases = getV1Releases;
//# sourceMappingURL=common.js.map