"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RubyGemsDatasource = void 0;
const tslib_1 = require("tslib");
const marshal_1 = require("@qnighy/marshal");
const http_1 = require("../../../util/http");
const result_1 = require("../../../util/result");
const url_1 = require("../../../util/url");
const rubyVersioning = tslib_1.__importStar(require("../../versioning/ruby"));
const datasource_1 = require("../datasource");
const common_1 = require("./common");
const http_2 = require("./http");
const metadata_cache_1 = require("./metadata-cache");
const schema_1 = require("./schema");
const versions_endpoint_cache_1 = require("./versions-endpoint-cache");
class RubyGemsDatasource extends datasource_1.Datasource {
    static id = 'rubygems';
    metadataCache;
    constructor() {
        super(RubyGemsDatasource.id);
        this.http = new http_2.RubygemsHttp(RubyGemsDatasource.id);
        this.versionsEndpointCache = new versions_endpoint_cache_1.VersionsEndpointCache(this.http);
        this.metadataCache = new metadata_cache_1.MetadataCache(this.http);
    }
    defaultRegistryUrls = ['https://rubygems.org'];
    defaultVersioning = rubyVersioning.id;
    registryStrategy = 'hunt';
    versionsEndpointCache;
    async getReleases({ packageName, registryUrl, }) {
        // istanbul ignore if
        if (!registryUrl) {
            return null;
        }
        const { val: rubygemsResult, err: rubygemsError } = await result_1.Result.wrap(this.versionsEndpointCache.getVersions(registryUrl, packageName))
            .transform((versions) => this.metadataCache.getRelease(registryUrl, packageName, versions))
            .unwrap();
        // istanbul ignore else: will be removed soon
        if (rubygemsResult) {
            return rubygemsResult;
        }
        else if (rubygemsError instanceof Error) {
            this.handleGenericErrors(rubygemsError);
        }
        try {
            const registryHostname = (0, url_1.parseUrl)(registryUrl)?.hostname;
            if (rubygemsError === 'unsupported-api' &&
                registryHostname !== 'rubygems.org') {
                if (registryHostname === 'rubygems.pkg.github.com' ||
                    registryHostname === 'gitlab.com') {
                    return await this.getReleasesViaFallbackAPI(registryUrl, packageName);
                }
                const { val: apiV1Result, err: apiV1Error } = await (0, common_1.getV1Releases)(this.http, registryUrl, packageName).unwrap();
                if (apiV1Result) {
                    return apiV1Result;
                }
                else if (apiV1Error instanceof http_1.HttpError) {
                    throw apiV1Error;
                }
                return await this.getReleasesViaFallbackAPI(registryUrl, packageName);
            }
            return null;
        }
        catch (error) {
            this.handleGenericErrors(error);
        }
    }
    async getReleasesViaFallbackAPI(registryUrl, packageName) {
        const path = (0, url_1.joinUrlParts)(registryUrl, `/api/v1/dependencies`);
        const query = (0, url_1.getQueryString)({ gems: packageName });
        const url = `${path}?${query}`;
        const { body: buffer } = await this.http.getBuffer(url);
        const data = marshal_1.Marshal.parse(buffer);
        return schema_1.MarshalledVersionInfo.parse(data);
    }
}
exports.RubyGemsDatasource = RubyGemsDatasource;
//# sourceMappingURL=index.js.map